/*
	This is a Virtual Window Manager for use under Win32 Shells that
	support the LiteStep module standard.

	Copyright (C) 2001 Chris Rempel

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <tchar.h>
#include <windows.h>

extern UINT nCurVW;
extern UINT nVWDesks;

BOOL AddBangCommand( LPCTSTR, void (__cdecl *)(struct HWND__ *, LPTSTR, ...) );
BOOL RemoveBangCommand( LPCTSTR );

VOID gatherToVW( UINT );
VOID switchToVW( UINT );

UINT getNextVW( UINT );
UINT getPrevVW( UINT );



/* Bang Commands */
VOID bVWMDesk( HWND caller, LPTSTR args, ... )
{
	if (args != NULL && *args != 0)
	{

		if (!_tcsicmp(_T(".home"), args))
			switchToVW( 0 );
		else if (!_tcsicmp(_T(".end"), args))
			switchToVW( nVWDesks-1 );
		else if (!_tcsicmp(_T(".next"), args))
			switchToVW( getNextVW(nCurVW) );
		else if (!_tcsicmp(_T(".prev"), args))
			switchToVW( getPrevVW(nCurVW) );
		else
			switchToVW( _ttoi(args) );

	}

	return;

		UNREFERENCED_PARAMETER( caller );
}

VOID bVWMNext( HWND caller, LPTSTR args, ... )
{
	switchToVW( getNextVW(nCurVW) );

	return;

		UNREFERENCED_PARAMETER( caller );
		UNREFERENCED_PARAMETER( args );
}

VOID bVWMPrev( HWND caller, LPTSTR args, ... )
{
	switchToVW( getPrevVW(nCurVW) );

	return;

		UNREFERENCED_PARAMETER( caller );
		UNREFERENCED_PARAMETER( args );
}

VOID bVWMGather( HWND caller, LPTSTR args, ... )
{
	if (args != NULL && *args != 0)
	{

		if (!_tcsicmp(_T(".home"), args))
			gatherToVW( 0 );
		else if (!_tcsicmp(_T(".end"), args))
			gatherToVW( nVWDesks-1 );
		else if (!_tcsicmp(_T(".next"), args))
			gatherToVW( getNextVW(nCurVW) );
		else if (!_tcsicmp(_T(".prev"), args))
			gatherToVW( getPrevVW(nCurVW) );
		else
			gatherToVW( _ttoi(args) );

	}
	else
		gatherToVW( nCurVW );

	return;

		UNREFERENCED_PARAMETER( caller );
}

VOID RegisterBangCmds( VOID )
{

	AddBangCommand(_T("!BVWMDESK"), bVWMDesk);
	AddBangCommand(_T("!BVWMNEXT"), bVWMNext);
	AddBangCommand(_T("!BVWMPREV"), bVWMPrev);
	AddBangCommand(_T("!BVWMGATHER"), bVWMGather);

	return;
}

VOID UnRegisterBangCmds( VOID )
{

	RemoveBangCommand("!BVWMDESK");
	RemoveBangCommand("!BVWMNEXT");
	RemoveBangCommand("!BVWMPREV");
	RemoveBangCommand("!BVWMGATHER");

	return;
}

