/*
	This is a Virtual Window Manager for use under Win32 Shells that
	support the LiteStep module standard.

	Copyright (C) 2001 Chris Rempel

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <tchar.h>
#include <windows.h>

#define SHELL_WINDOW_ID 0x49474541

#define VW_SPACING 10

#define VW_WIDTH (nDTWidth + VW_SPACING)
#define VW_HEIGHT (nDTHeight)

HWND hTopWnd;
UINT nCurVW, nVWDesks;
INT nDTWidth, nDTHeight;

INT GetRCInt( LPCTSTR, INT );

VOID LoadSettings( VOID )
{
	RECT r;

	GetClientRect(GetDesktopWindow(),&r);
	nDTWidth = r.right;
	nDTHeight = r.bottom;

	nVWDesks = GetRCInt( _T("bVWMDesks"), 2 );
	if (nVWDesks < 1)
		nVWDesks = 1;

	return;
}

UINT getNextVW( UINT vw )
{
	return( vw < nVWDesks-1 ? vw+1: 0 );
}

UINT getPrevVW( UINT vw )
{
	return( vw > 0 ? vw-1 : nVWDesks-1 );
}

BOOL CALLBACK EnumWindowsProc( HWND hwnd, LPARAM lParam )
{
	if (!(GetWindowLong(hwnd, GWL_EXSTYLE) & WS_EX_TOPMOST))
	{
		hTopWnd = hwnd;
		return FALSE;
	}
	return TRUE;
		UNREFERENCED_PARAMETER( lParam );
}

VOID switchToVW( UINT vw )
{
	if ( vw != nCurVW && vw >= 0 && vw < nVWDesks ){
		INT addH;
		HWND hwnd;
		HDWP dwp;
		RECT r;
		BOOL next;

		addH = ((vw % nVWDesks) - (nCurVW % nVWDesks)) * VW_WIDTH;

		next = FALSE;
		EnumWindows(EnumWindowsProc, 0);
		hwnd = GetWindow(hTopWnd, GW_HWNDLAST);

		dwp = BeginDeferWindowPos(1);

		do
		{

			if (next)
				hwnd = GetWindow(hwnd, GW_HWNDPREV);

			next = hwnd != NULL ? TRUE:FALSE;

			if (next)
			{
				if (!(GetWindowLong(hwnd, GWL_STYLE) & WS_VISIBLE))
					continue;
				if (GetWindowLong(hwnd, GWL_USERDATA) == SHELL_WINDOW_ID)
					continue;

				GetWindowRect(hwnd, &r);
				r.left-=addH;
				r.right-=addH;
				dwp = DeferWindowPos(dwp, hwnd, NULL, r.left, r.top, r.right-r.left, r.bottom-r.top, SWP_NOZORDER|SWP_NOACTIVATE);

			}

		}while (next);

		nCurVW = vw;

		EndDeferWindowPos(dwp);
	}

	return;
}

VOID gatherToVW( UINT vw )
{
	HWND hwnd;
	HDWP dwp;
	RECT r;
	BOOL next;
	INT width;

	nCurVW = vw;

	next = FALSE;
	EnumWindows(EnumWindowsProc, 0);
	hwnd = GetWindow(hTopWnd, GW_HWNDLAST);
	dwp = BeginDeferWindowPos(1);

	do
	{

		if (next)
			hwnd = GetWindow(hwnd, GW_HWNDPREV);

		next = hwnd != NULL ? TRUE:FALSE;

		if (next)
		{
			if (!(GetWindowLong(hwnd, GWL_STYLE) & WS_VISIBLE))
				continue;
			if ((GetWindowLong(hwnd, GWL_STYLE) & WS_MINIMIZE))
				continue;
			if (GetWindowLong(hwnd, GWL_USERDATA) == SHELL_WINDOW_ID)
				continue;

			GetWindowRect(hwnd, &r);

			width = abs(r.right - r.left);

				/* Initial x/width position calculation */
			if (r.left > nDTWidth)
			{
				r.left %= VW_WIDTH;
				if (r.left > nDTWidth)
					r.left -= VW_WIDTH;
				r.right = r.left + width;
			}
			else if (r.left < -(VW_SPACING))
			{
				r.left %= VW_WIDTH;
				while (r.left < -(VW_SPACING))
					r.left += VW_WIDTH;;
				r.right = r.left + width;
			}

				/* Don't allow a window to be in more then one VW at a time if possible. */
			if (r.right > VW_WIDTH)
			{
				if (r.left > r.right-nDTWidth)
					r.left -= (r.right-nDTWidth);
				else
				{
					if (width - VW_WIDTH < VW_SPACING)
						r.left = (width-nDTWidth)/-2;
					else
						r.left = 0;
				}
				r.right = r.left + width;
			}

			dwp = DeferWindowPos(dwp, hwnd, NULL, r.left, r.top, r.right-r.left, r.bottom-r.top, SWP_NOZORDER|SWP_NOACTIVATE);

		}

	}while (next);

	EndDeferWindowPos(dwp);

	return;
}